###########################################################################
add_path_items <- function(subdata) {
  last_event <- subdata$event[1]
  items <- unique(subdata$item)[unique(subdata$item) != "glossar"]
  n <- 1    # count items for progress

  pb <- utils::txtProgressBar(min = 0, max = nrow(subdata), initial = NA,
                       style = 3)

  for (item in items) {

    cat("\n\nAdding path variable for item", item,
               paste0("(", n, "/", length(items), ")"), "\n")

    for (i in 1:nrow(subdata)) {
      if (last_event == "Show Info" & subdata$item[i] == item) {
        subdata$path[i] <- i
        j <- i
      } else if (last_event == "Show Front" & subdata$item[i] == item) {
        subdata$path[i] <- j
      } else if (!(last_event %in% c("Show Info", "Show Front")) &
                 subdata$item[i] == item) {
        subdata$path[i] <- j
      }
      if (i <= nrow(subdata)) {
        last_event <- subdata$event[i + 1]
      }
      utils::setTxtProgressBar(pb, i)
    }
    n <- n + 1
  }
  subdata
}

###########################################################################
add_path_glossar <- function(subdata, xmlpath) {

  pb <- utils::txtProgressBar(min = 0, max = nrow(subdata), initial = NA,
                       style = 3)

  cat("\n\n########## Creating glossar dictionary ##########", "\n")
  items      <- unique(subdata$item[subdata$item != "glossar"])
  glossar_files <- unique(subdata[subdata$item == "glossar", "popup"])
  lut           <- create_glossardict(items, glossar_files, xmlpath = xmlpath)
  inside        <- glossar_files[glossar_files %in%
                                 names(lut[sapply(lut, length) == 1])]
  single_art    <- unlist(lut[names(lut) %in% inside])

  m <- 1

  for (file in names(lut)) {

    cat("\n\nAdding path variable for glossar entry", file,
               paste0("(", m, "/", length(lut), ")"), "\n")

    item_list <- unlist(lut[names(lut) == file])

    for (i in seq_len(nrow(subdata))) {
      if (subdata$event[i] == "Show Info" |
          (subdata$event[i] == "Artwork/OpenCard" &
           subdata$item[i] %in% single_art)) {
        current_item <- subdata[i, "item"]
        j <- i
        k <- i
      } else {
        current_item <- current_item
      }
      if (subdata$event[i] == "Show Front" & subdata$item[i] == current_item) {
      # make sure item has not been closed, yet!
        k <- i
      }
      if (subdata$item[i] == "glossar" &
          (current_item %in% item_list) &
          subdata$popup[i] == file & (j - k == 0)) {
        subdata[i, "path"]   <- subdata[j, "path"]
        subdata[i, "item"] <- current_item
      }
      utils::setTxtProgressBar(pb, i)
    }
    m <- m + 1
  }

  # Exclude not matched glossar entries
  cat("\n\nINFORMATION: glossar entries that are not matched will be removed:",
      sum(is.na(subdata[subdata$glossar == 1, "path"])), "entries",
      fill = TRUE)
  subset(subdata, !is.na(subdata$path))
}

###########################################################################
add_path <- function(data, xmlpath, glossar) {

  data$path <- NA
  subdata1 <- data[data$event %in% c("Transform start", "Transform stop"), ]
  subdata2 <- data[!data$event %in% c("Transform start", "Transform stop"), ]

  subdata2 <- add_path_items(subdata2)

  if (glossar) {
    subdata2 <- add_path_glossar(subdata2, xmlpath)
  } else {
    subdata2 <- subdata2[subdata2$glossar != 1, ]
  }

  out <- rbind(subdata1, subdata2)
  out <- out[order(out$fileId, out$date, out$timeMs), ]
  out
}

###########################################################################

# Add path for moves

add_path_moves <- function(data) {

  pbapply::pboptions(style = 3, char = "=")

  path_max <- max(data$path, na.rm = TRUE)

  #subdata_art <- split(data, ~ item)
  subdata_case <- split(data, ~ case)

  #subdata_list <- split(data, ~ item + case)
  # --> does not work with complete data set
  cat("Splitting data...", "\n")
  subdata_list <- pbapply::pblapply(subdata_case, split, f = ~item)
  subdata_list <- unlist(subdata_list, recursive = FALSE)

  cat("Adding path...", "\n")
  subdata_path <- pbapply::pblapply(subdata_list,
        function(x) {
          path_max <<- path_max + 1
          add_path_subdata(x, max_path = path_max)
        }
      )

  out <- dplyr::bind_rows(subdata_path)
  out <- out[order(out$fileId.start, out$date.start, out$timeMs.start), ]
  rownames(out) <- NULL

  # Make path a consecutive number
  out$path <- as.numeric(factor(out$path, levels = unique(out$path)))
  out
}


add_path_subdata <- function(subdata, max_path) {

  if (nrow(subdata) != 0) {

    if (length(stats::na.omit(unique(subdata$path))) == 1) {
      subdata[subdata$event == "move", "path"] <- stats::na.omit(unique(subdata$path))
    } else if (length(stats::na.omit(unique(subdata$path))) > 1) {
      for (i in 1:nrow(subdata)) {
        if (subdata$event[i] == "move") {
          if (i == 1) {
            subdata$path[i] <- stats::na.omit(unique(subdata$path))[1]
          } else {
            subdata$path[i] <- subdata$path[i - 1]
          }
        }
      }
    } else if (all(is.na(subdata$path))) {
      for (i in 1:nrow(subdata)) {
        subdata$path[i] <- max_path
      }
    }

  } else {
    warning("subdata has nrow = 0")
  }
  subdata
}

